/*
      ___           ___           ___                        ___     
     /  /\         /__/\         /  /\          ___         /  /\    
    /  /:/_        \  \:\       /  /::\        /__/\       /  /:/_   
   /  /:/ /\        \__\:\     /  /:/\:\       \  \:\     /  /:/ /\  
  /  /:/ /::\   ___ /  /::\   /  /:/  \:\       \  \:\   /  /:/ /:/_ 
 /__/:/ /:/\:\ /__/\  /:/\:\ /__/:/ \__\:\  ___  \__\:\ /__/:/ /:/ /\
 \  \:\/:/~/:/ \  \:\/:/__\/ \  \:\ /  /:/ /__/\ |  |:| \  \:\/:/ /:/
  \  \::/ /:/   \  \::/       \  \:\  /:/  \  \:\|  |:|  \  \::/ /:/ 
   \__\/ /:/     \  \:\        \  \:\/:/    \  \:\__|:|   \  \:\/:/  
     /__/:/       \  \:\        \  \::/      \__\::::/     \  \::/   
     \__\/         \__\/         \__\/           ~~~~       \__\/    
                        
			    ___           ___   
			   /  /\         /  /\  
			  /  /:/        /  /:/  
			 /__/::\       /  /:/   
			 \__\/\:\__   /  /::\   
			    \  \:\/\ /__/:/\:\  
			     \__\::/ \__\/  \:\ 
			     /__/:/       \  \:\
			     \__\/         \__\/

*/

#include "aux_functions.h"

/*----------------------------------------------------------------------------*/

int empty_list(list *l)
{
    return(l->size == 0);
}

/*----------------------------------------------------------------------------*/

list *build_list (void)
{
    list *l = malloc(sizeof(list));
    if (l == NULL)
        return (NULL);

    l->first = l->last = NULL;
    l->size = 0;

    return (l);
}

/*----------------------------------------------------------------------------*/

node *store_list(list *l, int direction)
{
    node *new = malloc(sizeof(node));
    if (new == NULL)
        return (NULL);

    new->direction = direction;
    new->next = NULL;

    if(empty_list(l))
    {
        l->last  = malloc(sizeof(node));
        l->first = new;
    }

    l->last->next = new;
    l->last = new;
    l->size++;

    //print_list (l);

    return (new);
}

/*----------------------------------------------------------------------------*/

void print_list(list *l)
{
    int i = 0;;
    node *n = l->first;;
    
    acquire_screen();

    while (n != NULL)
    {
        textprintf_ex(screen, font, i*30+100, 100, makecol(255,255,255), makecol(0,0,0), "'%d',", n->direction);
        n = n->next;
        i++;
    }

    release_screen();
}

/*----------------------------------------------------------------------------*/

void destroy_list (list *l)
{
    node *destroying;

    while ( (destroying = l->first) )
    {        
        l->first = l->first->next;
        free(destroying);
    }

    free(l);
    l=NULL;
}

/*----------------------------------------------------------------------------*/

void remove_last_node (list *l)
{
    if(l->first == l->last)
    {
        destroy_list(l);
        l = build_list();
        return;
    }

    node *n = l->first;

    while ( n->next != l->last )
        n = n->next;
    
    n->next = n->next->next;
    l->last = n;

    free(n->next);

    l->size--;
}

/*---------------------------------------------------------------------------*/

field *load_stage(char *stage_to_open, field *stage)
{

	int lines, columns;
    char place;
	int i,j;
	FILE *stage_file;

	acquire_screen();
	clear_to_color(screen, makecol(0,0,0));

	textprintf_ex (screen, font, 15, 15, makecol(255, 255, 255), -1,
				   "trying to load stage '%s'",stage_to_open);

	release_screen();

	stage->right_places = NULL;
	stage->field = NULL;
	stage->level = atoi(stage_to_open);

	stage_file = fopen (stage_to_open, "r");

	if (stage_file == NULL) //error opening file
		error_opening_stage (stage_to_open);


	//allocating memory
	fscanf (stage_file, "%d", &lines);
	fscanf (stage_file, "%d", &columns);

	stage->field = allocate_matrix (lines,columns);
	if (stage->field == NULL)
		error_allocating_memory ("error_1_load_stage");

	//loadind player's begining position
	fscanf (stage_file, "%d", &stage->player.x);
	fscanf (stage_file, "%d", &stage->player.y);

	//loading places where boxes are supposed to be at
	fscanf (stage_file, "%d", &stage->number_of_right_places);

	stage->right_places = malloc
						  (stage->number_of_right_places * sizeof (position));

	if (stage->right_places == NULL)
		error_allocating_memory ("error_2_load_stage");

	for (i=0; i<stage->number_of_right_places; i++)
	{
		fscanf (stage_file, "%d", &stage->right_places[i].x);
		fscanf (stage_file, "%d", &stage->right_places[i].y);
	}

	//filling field from file
	for (i=0; i<lines; i++)
		for (j=0; j<columns; j++)
        {
            fscanf(stage_file, "%c", &place);
            if (place == 'x' || place == '.' || place == ' ' || place == '0' || place == '#')
			    stage->field[i][j-1] = place;
        }
    //stage->number_of_steps = 0;
    
	fclose (stage_file);
	stage_file = NULL;

	stage->lines = lines;
	stage->columns = columns;


    //printing stage on screen
	acquire_screen();
	textprintf_ex (screen, font, 15, 35, makecol(255, 255, 255), -1, "loaded");


    draw_sprite(screen, back, 0, 0);

    //displays walls and empty spaces image
    for (i=0; i<stage->lines; i++)
        for (j=0; j<stage->columns-1; j++)
            switch (stage->field[i][j])
            {
                 case BOX :
                     draw_sprite(screen, box, j*GAP,
                             i*GAP);
                     break;
                 case 'x' :
                     draw_sprite(screen, wall, j*GAP, i*GAP);
                      break;
                  case ' ' :
                      draw_sprite(screen, bkg, j*GAP, i*GAP);
                      break;
                  default :
                      break;
            }

     //displays right places image
     for (i=0; i<stage->number_of_right_places; i++)
         if (stage->field [stage->right_places[i].x][stage->right_places[i].y]
             != BOX)

             draw_sprite(screen, right_place, stage->right_places[i].y*GAP,
                         stage->right_places[i].x*GAP);
 
         else
             draw_sprite(screen, box_at_place, stage->right_places[i].y*GAP,
                         stage->right_places[i].x*GAP);

    //draw_sprite(screen, player[12], 10+ stage->player.y*GAP -GAP, 10+ stage->player.x*GAP);
    draw_sprite(screen, player[12], 10+ stage->player.y*GAP, 10+ stage->player.x*GAP);

	release_screen();
    //done printing

	return (stage);
}

/*----------------------------------------------------------------------------*/

char *which_stage_to_load(char *password)
{
	if (strcmp (password, "marina!!") == 0 )
		return ("11");

	if (strcmp (password, "malibu") == 0 )
        return("21");

	if (strcmp (password, "sanpedro") == 0 )
        return("31");

	if (strcmp (password, "ventura?") == 0 )
        return("41");

	if (strcmp (password, "sandiego") == 0 )
        return("51");

	if (strcmp (password, "pasadena") == 0 )
        return("61");

	if (strcmp (password, "!beverly") == 0 )
        return("71");

	if (strcmp (password, "melrose!") == 0 )
        return("81");

	if (strcmp (password, "rodeo!?!") == 0 )
        return("91");

	if (strcmp (password, "westwood") == 0 )
        return("101");

	if (strcmp (password, "wilshire") == 0 )
        return("111");

	if (strcmp (password, "venice??") == 0 )
        return("121");

	if (strcmp (password, "figueroa") == 0 )
        return("131");

	if (strcmp (password, "sunset") == 0 )
        return("141");

	if (strcmp (password, "orange") == 0 )
        return("151");

	if (strcmp (password, "!ayasam!") == 0 )
        return("0");


	return (NULL);
}

/*----------------------------------------------------------------------------*/

//here we keep stage->level ;)
void unload_stage(field *stage)
{
	int i;

    if(stage)
    {
    	//unloading matrix
    	for (i=0; i<stage->lines; i++)
            if(stage->field[i])
        		free (stage->field[i]);
        if(stage->field)        
        	free (stage->field);

    	stage->lines = 0;
    	stage->columns = 0;

    	stage->player.x = 0;
    	stage->player.y = 0;

        if(stage->right_places)
        	free (stage->right_places);
    }
}

/*----------------------------------------------------------------------------*/

char **allocate_matrix(int lines, int columns)
{
	int i;
	char **matrix;

	matrix = malloc (lines * sizeof (char *));

	if (matrix == NULL)
		return (NULL);

	for (i=0; i<lines; i++)
	{
		matrix[i] = malloc (columns * sizeof (char));
		if (matrix[i] == NULL)
			return (NULL);
	}

	return (matrix);
}

/*----------------------------------------------------------------------------*/

void read_string(char *str_to, int size)
{
        int cur_pos = 0; // position of caret
        int the_key = 0;
        int i;

        for (i = 0; i < size; i++)
        str_to[i] = '\0'; // 'clean' the string

        while (the_key>>8 != KEY_ENTER)
        {
                the_key = readkey();

                if ((the_key & 0xff) >= ' ') // get only valid chars
                {
                        str_to[cur_pos] = the_key & 0xff;
                        cur_pos++;
                        if (cur_pos > size-2) cur_pos = size-2;

                }

                if (the_key >> 8 == KEY_BACKSPACE)
                {
                        str_to[cur_pos] = '\0'; // chop the string
                        cur_pos --;
                        if (cur_pos < 0) cur_pos = 0;
                }

                // lame redraw (use double buffer, whatever)
                clear(screen);
                textout_ex(screen,font, str_to, 8,1, makecol(255,255,255),
			   makecol(0,0,0));

        }

}

/*----------------------------------------------------------------------------*/

char *int_to_string (int number)
{
	char *str;
	int chars;
	int aux,i;

	aux = number;
	chars = 1;

	while ( aux >= 10)
	{
		chars++;
		aux /= 10;
	}

	str = malloc ( chars * sizeof(char) );

	//this was not supposed to be necessary, not even work ...
	str[chars] = '\0'; // 'clean' the string

	chars--;
	aux = number;

	i=0;
	while ( aux >= 10)
	{
		str[chars--] = aux%10+48;
		i+=5;
		aux /= 10;
	}

	str[0] = aux+48;

	return (str);

}

/*----------------------------------------------------------------------------*/

int all_boxes_placed (field *stage)
{
	int i,j,k;
	int placed = 0;

	for (k=0; k <= stage->number_of_right_places; k++)

		for (i=0; i<stage->lines; i++)
			for (j=0; j<stage->columns; j++)
				if ( (stage->field[i][j] == BOX) &&
				    (i == stage->right_places[k].x &&
					 j == stage->right_places[k].y) )
					{
						placed++;
						break;
					}

	if (placed == stage->number_of_right_places)
		return (1);

	return (0);
}

/*----------------------------------------------------------------------------*/

int is_right_place(field *stage, int x, int y)
{
    int k;

    for (k=0; k <= stage->number_of_right_places; k++)
        if ((stage->right_places[k].x == x) && (stage->right_places[k].y == y))
            return (1);

    return (0);
}

/*----------------------------------------------------------------------------*/

field *choosing_level(int level)
{
    int range = 0;
    field *stage = malloc(sizeof(field));

    stage = load_stage(int_to_string(level), stage);

    while(!key[KEY_ENTER])
    {//0-8 has 9 number, rage == 9, range + 1 == 10 possible levels to load
        if(key[KEY_LEFT] && range>0)
            stage = load_stage(int_to_string(--range + level), stage);
        if(key[KEY_RIGHT] && range<=8)
            stage = load_stage(int_to_string(++range + level), stage);
        if(key[KEY_ESC])
            finish_game();
    }
    
    return stage;
}


/*----------------------------------------------------------------------------*/

void finish_game()
{
	exit (0);
}
